[![npm version](https://badge.fury.io/js/vue-cli-plugin-html-replace.svg)](https://badge.fury.io/js/vue-cli-plugin-html-replace)
[![GitHub license](https://img.shields.io/github/license/tpavard/vue-cli-plugin-html-replace.svg)](https://github.com/tpavard/vue-cli-plugin-html-replace/blob/master/LICENSE)
[![GitHub issues](https://img.shields.io/github/issues/tpavard/vue-cli-plugin-html-replace.svg)](https://github.com/tpavard/vue-cli-plugin-html-replace/issues)


# Vue-cli Plugin HTML Replace

A simple plugin to replace HTML contents within templates that are generated by [vue-cli](https://github.com/vuejs/vue-cli).

## Getting started

:warning: Make sure you have [vue-cli 3.x.x](https://github.com/vuejs/vue-cli) installed:

```
vue -V
```

Navigate to your vue app folder and add the cli plugin:

```
vue add html-replace
```

## Basic usage

This plugin can be configured by defining `htmlReplace` via the `pluginOptions` in your `vue.config.js`.

The `patterns` argument should be either an object or an array of objects. Each pattern should contain both `pattern` and `replacement` options.
The `enable` parameter is optional.

```javascript
module.exports = {
    pluginOptions: {
        htmlReplace: {
            enable: (process.env.NODE_ENV === "production"),
            patterns: [
                {
                    pattern: "foo",
                    replacement: "bar",
                },
                {
                    pattern: /foo/g,
                    replacement: "bar",
                },
                {
                    pattern: /<title>(.*)<\/title>/,
                    replacement: (match, $1) => `<title>${$1} has been replaced.</title>`,
                },
            ],
        },
    },
};
```

## Multi-page mode

In case you'll need to prevent some patterns to be applied to some of your pages, you can specify one of either `excludes` or `includes` options. Each entries should be named after the key given for each page. These parameters are optional and will be ignored if you're not building your app in multi-page mode. When not defined in multi-page mode, the patterns are applied to all the pages.

```javascript
module.exports = {
    pages: {
        index: {
            entry: "src/App_1/main.js",
            filename: "app_1.html"
        },
        app_2: "src/App_2/main.js",
        app_3: "src/App_3/main.js",
    },
    pluginOptions: {
        htmlReplace: {
            patterns: [
                {
                    pattern: "foo",
                    replacement: "this pattern will be applied to every page",
                },
                {
                    pattern: /<title>.*<\/title>/,
                    replacement: "<title>this is the index page</title>",
                    includes: "index",
                },
                {
                    pattern: /<title>.*<\/title>/,
                    replacement: "<title>this is the second page</title>",
                    excludes: ["index" , "app_3"],
                },
            ],
        },
    },
};
```

## API

- __patterns__ `object | [ object1[, object2[, ...[, objectN ]]]]` Adds some patterns and defines how to replace the match(es). Can be a single object or a list of objects.
    - __pattern__ `string | RegExp` A RegExp or string. The match or matches are replaced with the `replacement` value.
    - __replacement__ `string | function` The string or function that replaces the substring by the specified `pattern` parameter.
    - __excludes__ _optional_ `string | [ string1[, string2[, ...[, stringN ]]]]` A string or array of strings to exclude some pages for which the pattern would be applied to.
    - __includes__ _optional_ `string | [ string1[, string2[, ...[, stringN ]]]]` A string or array of strings to include some pages for which the pattern would be applied to.
- __enable__ _optional_ `(true) | false` Enable/disable the plugin.